@@ -28,7 +28,7 @@ use hir::{AsAssocItem, FieldSource, HasSource, InFile, ModuleSource, Semantics};
 use stdx::never;
 use syntax::{
     ast::{self, HasName},
-    lex_single_syntax_kind, AstNode, SyntaxKind, TextRange, T,
+    AstNode, SyntaxKind, TextRange, T,
 };
 use text_edit::{TextEdit, TextEditBuilder};
 
@@ -67,10 +67,8 @@ pub use _bail as bail;
 impl Definition {
     pub fn rename(&self, sema: &Semantics<RootDatabase>, new_name: &str) -> Result<SourceChange> {
         match *self {
-            Definition::ModuleDef(hir::ModuleDef::Module(module)) => {
-                rename_mod(sema, module, new_name)
-            }
-            Definition::ModuleDef(hir::ModuleDef::BuiltinType(_)) => {
+            Definition::Module(module) => rename_mod(sema, module, new_name),
+            Definition::BuiltinType(_) => {
                 bail!("Cannot rename builtin type")
             }
             Definition::SelfType(_) => bail!("Cannot rename `Self`"),
@@ -101,26 +99,22 @@ impl Definition {
                     FieldSource::Pos(_) => None,
                 }
             }
-            Definition::ModuleDef(module_def) => match module_def {
-                hir::ModuleDef::Module(module) => {
-                    let src = module.declaration_source(sema.db)?;
-                    let name = src.value.name()?;
-                    src.with_value(name.syntax()).original_file_range_opt(sema.db)
-                }
-                hir::ModuleDef::Function(it) => name_range(it, sema),
-                hir::ModuleDef::Adt(adt) => match adt {
-                    hir::Adt::Struct(it) => name_range(it, sema),
-                    hir::Adt::Union(it) => name_range(it, sema),
-                    hir::Adt::Enum(it) => name_range(it, sema),
-                },
-                hir::ModuleDef::Variant(it) => name_range(it, sema),
-                hir::ModuleDef::Const(it) => name_range(it, sema),
-                hir::ModuleDef::Static(it) => name_range(it, sema),
-                hir::ModuleDef::Trait(it) => name_range(it, sema),
-                hir::ModuleDef::TypeAlias(it) => name_range(it, sema),
-                hir::ModuleDef::BuiltinType(_) => return None,
+            Definition::Module(module) => {
+                let src = module.declaration_source(sema.db)?;
+                let name = src.value.name()?;
+                src.with_value(name.syntax()).original_file_range_opt(sema.db)
+            }
+            Definition::Function(it) => name_range(it, sema),
+            Definition::Adt(adt) => match adt {
+                hir::Adt::Struct(it) => name_range(it, sema),
+                hir::Adt::Union(it) => name_range(it, sema),
+                hir::Adt::Enum(it) => name_range(it, sema),
             },
-            Definition::SelfType(_) => return None,
+            Definition::Variant(it) => name_range(it, sema),
+            Definition::Const(it) => name_range(it, sema),
+            Definition::Static(it) => name_range(it, sema),
+            Definition::Trait(it) => name_range(it, sema),
+            Definition::TypeAlias(it) => name_range(it, sema),
             Definition::Local(local) => {
                 let src = local.source(sema.db);
                 let name = match &src.value {
@@ -150,6 +144,10 @@ impl Definition {
                 let lifetime = src.value.lifetime()?;
                 src.with_value(lifetime.syntax()).original_file_range_opt(sema.db)
             }
+            Definition::BuiltinType(_) => return None,
+            Definition::SelfType(_) => return None,
+            Definition::BuiltinAttr(_) => return None,
+            Definition::ToolModule(_) => return None,
         };
         return res;
 
@@ -190,17 +188,24 @@ fn rename_mod(
         source_change.push_file_system_edit(move_file);
     }
 
-    if let Some(InFile { file_id, value: decl_source }) = module.declaration_source(sema.db) {
-        let file_id = file_id.original_file(sema.db);
-        match decl_source.name() {
-            Some(name) => source_change.insert_source_edit(
-                file_id,
-                TextEdit::replace(name.syntax().text_range(), new_name.to_string()),
-            ),
+    if let Some(src) = module.declaration_source(sema.db) {
+        let file_id = src.file_id.original_file(sema.db);
+        match src.value.name() {
+            Some(name) => {
+                if let Some(file_range) =
+                    src.with_value(name.syntax()).original_file_range_opt(sema.db)
+                {
+                    source_change.insert_source_edit(
+                        file_id,
+                        TextEdit::replace(file_range.range, new_name.to_string()),
+                    )
+                };
+            }
             _ => never!("Module source node is missing a name"),
         }
     }
-    let def = Definition::ModuleDef(hir::ModuleDef::Module(module));
+
+    let def = Definition::Module(module);
     let usages = def.usages(sema).all();
     let ref_edits = usages.iter().map(|(&file_id, references)| {
         (file_id, source_edit_from_references(references, def, new_name))
@@ -239,35 +244,40 @@ fn rename_reference(
         }
     }
 
-    def = match def {
+    let assoc_item = match def {
         // HACK: resolve trait impl items to the item def of the trait definition
         // so that we properly resolve all trait item references
-        Definition::ModuleDef(mod_def) => mod_def
-            .as_assoc_item(sema.db)
-            .and_then(|it| it.containing_trait_impl(sema.db))
-            .and_then(|it| {
-                it.items(sema.db).into_iter().find_map(|it| match (it, mod_def) {
-                    (hir::AssocItem::Function(trait_func), hir::ModuleDef::Function(func))
+        Definition::Function(it) => it.as_assoc_item(sema.db),
+        Definition::TypeAlias(it) => it.as_assoc_item(sema.db),
+        Definition::Const(it) => it.as_assoc_item(sema.db),
+        _ => None,
+    };
+    def = match assoc_item {
+        Some(assoc) => assoc
+            .containing_trait_impl(sema.db)
+            .and_then(|trait_| {
+                trait_.items(sema.db).into_iter().find_map(|it| match (it, assoc) {
+                    (hir::AssocItem::Function(trait_func), hir::AssocItem::Function(func))
                         if trait_func.name(sema.db) == func.name(sema.db) =>
                     {
-                        Some(Definition::ModuleDef(hir::ModuleDef::Function(trait_func)))
+                        Some(Definition::Function(trait_func))
                     }
-                    (hir::AssocItem::Const(trait_konst), hir::ModuleDef::Const(konst))
+                    (hir::AssocItem::Const(trait_konst), hir::AssocItem::Const(konst))
                         if trait_konst.name(sema.db) == konst.name(sema.db) =>
                     {
-                        Some(Definition::ModuleDef(hir::ModuleDef::Const(trait_konst)))
+                        Some(Definition::Const(trait_konst))
                     }
                     (
                         hir::AssocItem::TypeAlias(trait_type_alias),
-                        hir::ModuleDef::TypeAlias(type_alias),
+                        hir::AssocItem::TypeAlias(type_alias),
                     ) if trait_type_alias.name(sema.db) == type_alias.name(sema.db) => {
-                        Some(Definition::ModuleDef(hir::ModuleDef::TypeAlias(trait_type_alias)))
+                        Some(Definition::TypeAlias(trait_type_alias))
                     }
                     _ => None,
                 })
             })
             .unwrap_or(def),
-        _ => def,
+        None => def,
     };
     let usages = def.usages(sema).all();
 
@@ -340,6 +350,10 @@ fn source_edit_from_name_ref(
     new_name: &str,
     def: Definition,
 ) -> bool {
+    if name_ref.super_token().is_some() {
+        return true;
+    }
+
     if let Some(record_field) = ast::RecordExprField::for_name_ref(name_ref) {
         let rcf_name_ref = record_field.name_ref();
         let rcf_expr = record_field.expr();
@@ -487,7 +501,7 @@ pub enum IdentifierKind {
 
 impl IdentifierKind {
     pub fn classify(new_name: &str) -> Result<IdentifierKind> {
-        match lex_single_syntax_kind(new_name) {
+        match parser::LexedStr::single_token(new_name) {
             Some(res) => match res {
                 (SyntaxKind::IDENT, _) => Ok(IdentifierKind::Ident),
                 (T![_], _) => Ok(IdentifierKind::Underscore),
